local L = Capitalist.Locale
local realm = GetRealmName()
local character = UnitName("player")

function Capitalist:StartScan()
	self:DebugEntry("StartScan")

	if (self.scanBuckets and
		#self.scanBuckets > 0) then

		self.bucketIndex = 1
		QueryAuctionItems(self.scanBuckets[self.bucketIndex],nil,nil,0,0,0,0,0,0,nil)
		self:DebugMisc(self.scanBuckets[self.bucketIndex])

		self.page = 0
	
		self.auctionUI.lastScanLabel:Hide()
		self.auctionUI.progressLabel:Show()
		self.auctionUI.progressLabel:SetText(L["Scanning"].."...")
		self.auctionUI.progress:Show()
		self.auctionUI.progress:SetWidth(1)
		self.auctionUI.scanButton:Disable()
		self.auctionUI.postButton:Disable()
	
		for productName, data in pairs(self.list) do
			data.ownAuctions = 0
			data.otherAuctions = {}

			-- calculate cost of the product
			data.cost = 0
			for componentName, componentCount in pairs(self.db.profile.professions[data.profession][data.skillName].components) do
				local componentPrice = self:GetComponentPrice(componentName)
				data.cost = data.cost + componentCount * componentPrice * 10000
			end
		
			-- add auction house cut to the price and minProfit (thanks - why did I forget that again?)
			data.cost = self.db.profile.minProfit + data.cost / 0.95 

			-- calculate maximum price
			data.bestPrice = data.cost * (1+self.db.profile.margin/100)
			if (data.bestPrice - data.cost < self.db.profile.minMargin) then
				data.bestPrice = data.cost + self.db.profile.minMargin
			end
			if (data.bestPrice - data.cost > self.db.profile.maxMargin) then
				data.bestPrice = data.cost + self.db.profile.maxMargin
			end
			data.bestPrice = math.floor(data.bestPrice) -- kills problems with float numbers
		end
	
		self.mode = "scanning"
		self.updateHandler:Show()
	end

	self:DebugExit("StartScan")
end

function Capitalist:OnUpdateScanning()
	self:DebugEntry("OnUpdateScanning")
	if (CanSendAuctionQuery()) then
		self.page = self.page + 1
		self:Scan()
	end
	self:DebugExit("OnUpdateScanning")
end

function Capitalist:Scan()
	self:DebugEntry("Scan")
	self.auctionUI.progressLabel:SetText(string.format(L["Scanning %d/%d"], self.bucketIndex, #self.scanBuckets))
	self:DebugMisc(self.scanBuckets[self.bucketIndex])
	QueryAuctionItems(self.scanBuckets[self.bucketIndex],nil,nil,0,0,0,self.page,0,0,nil)
	
	for i = 1, 50 do
		local name, _, _, _, _, _, _, _, buyoutPrice, _, _, owner, _  = GetAuctionItemInfo("list", i);

		if (name == nil) then
			if (self.bucketIndex < #self.scanBuckets) then
				self.bucketIndex = self.bucketIndex + 1
				-- REVIEW: trick so that next call to scan yields "self.page" of 0
				self.page = -1
				break
			else
				self:ScanFinished()

				-- REVIEW: single point of return
				return self:DebugExit("Scan")
			end
		end

		-- TODO: Add concept of "my" w/r/t "same account" auctions

		local v = self.list[name]
		if (v) then
			if (owner == character) then
				v.ownAuctions = v.ownAuctions + 1
				if (v.buyoutSelf == nil or buyoutPrice < v.buyoutSelf) then
					v.buyoutSelf = buyoutPrice
				end
			else
				table.insert(v.otherAuctions, buyoutPrice)
				if (v.buyoutOthers == nil or buyoutPrice < v.buyoutOthers) then
					v.buyoutOthers = buyoutPrice
					v.otherOwner = owner
				end
			end
		end
	end
	
	local numBatchAuctions, totalAuctions = GetNumAuctionItems("list");

	self.auctionUI.progress:SetWidth(100 * self.bucketIndex / #self.scanBuckets)

	self:DebugExit("Scan")
end

function Capitalist:ScanFinished()
	self:DebugEntry("ScanFinished")
	self.updateHandler:Hide()

	self.auctionUI.lastScanLabel:SetText(L["Last Scan: "]..date("%H:%M"))
	self.auctionUI.lastScanLabel:Show()
	self.auctionUI.progress:Hide()
	self.auctionUI.progressLabel:Hide()
	self.auctionUI.scanButton:Enable()
	
	-- consistency check
	for i = 1, GetNumAuctionItems("owner") do
		local name, _, _, _, _, _, _, _, _, _, _, _, saleStatus = GetAuctionItemInfo("owner", i);
		if (saleStatus == 0 and self.list[name]) then
			self.list[name].ownAuctions = self.list[name].ownAuctions - 1
		end
	end
	
	for k, v in pairs(self.list) do
		if (v.ownAuctions ~= 0) then
			self:DebugMisc(k)
			self:DebugMisc(v.profession)
			self:DebugMisc(v.skillName)
			self:DebugMisc(v.ownAuctions)

			self.auctionUI.lastScanLabel:SetText(L["Last Scan: failed"])
			self.validScan = false
			return
		end
	end

	-- calculating optimal prices by undercutting competition
	for k, v in pairs(self.list) do
		for j = 1, #v.otherAuctions do
			if (v.otherAuctions[j] <= v.bestPrice) then
				if (v.otherAuctions[j] > v.cost) then
					v.bestPrice = v.otherAuctions[j] - 1
				end
			end
		end
	end

	self.validScan = true
	self:UpdateAuctionUI()
	self.auctionUI.postButton:Enable()
	self:CheckForCancel()
	self:DebugExit("ScanFinished")
end

function Capitalist:CheckForCancel()
	self.toCancel = {}

	for i = 1, GetNumAuctionItems("owner") do
		local name, _, _, _, _, _, _, _, buyoutPrice, bidAmount, _, _, _ = GetAuctionItemInfo("owner", i);
		if (self.list[name] and bidAmount == 0) then
			if (buyoutPrice > self.list[name].bestPrice and self.db.profile.cancelYellow) then
				table.insert(self.toCancel, i)
			elseif (buyoutPrice < self.list[name].bestPrice and self.db.profile.cancelBlue) then
				table.insert(self.toCancel, i)
			elseif (self.db.profile.cancelDuration and GetAuctionItemTimeLeft("owner", i) ~= 4) then
				table.insert(self.toCancel, i)
			end
		end
	end

	if (#self.toCancel > 0) then
		self.auctionUI.cancelButton:Enable()
	else
		self.auctionUI.cancelButton:Disable()
	end
end

function Capitalist:CancelAuctions()
	self:DebugEntry("CancelAuctions")
	for i = 1, table.getn(self.toCancel) do
		CancelAuction(self.toCancel[i])
	end
	self.toCancel = {}
	self.auctionUI.cancelButton:Disable()
	self:DebugExit("CancelAuctions")
end
